import java.io.*;
import java.util.Enumeration;
import java.util.Vector;

import org.kxml.*;
import org.kxml.io.XmlWriter;
import org.kxml.kdom.Document;
import org.kxml.kdom.Element;
import org.kxml.parser.AbstractXmlParser;
import org.kxml.parser.XmlParser;
import org.kxml.wap.WbxmlWriter;

/**
 * Demonstrate xpath
 */
public class XPathDemo {

	// Reference to standard out.
	private static final PrintStream stdout = System.out;

	/**
	* Starts the parse of an XML document contained in the given file.
	*/
	public void performDemo(String file, String xpath) {
		FileReader fin = null;
		StringWriter s = new StringWriter();
		
		try {
			fin = new FileReader(file);
			XmlParser parser = new XmlParser(fin);
			Document document = new Document();
			document.parse(parser);
			Element root = document.getRootElement();
			XPathExpression xpls = new XPathExpression(document, xpath);
         
			Vector result = xpls.getResult();
         
			for(Enumeration e = result.elements(); e.hasMoreElements(); ) {
				Object obj = e.nextElement();
			 	if(obj instanceof Element)
			 		processNode(obj, Xml.ELEMENT);
			 	else //can only be string
			 		processNode(obj, Xml.TEXT);         
			}
		} catch (Exception ex) {
			stdout.println("***Exception Occurred***");
			stdout.println(ex.getMessage());
			ex.printStackTrace(stdout);
		} finally {
			if (fin != null) {
				try {
					fin.close();
				} catch (IOException ex) {
					stdout.println("***Exception Occurred***");
					stdout.println(ex.getMessage());
					ex.printStackTrace(stdout);
				}
			}
		}         
	}

	// Outputs information for the given node and its type.
	private void processNode(Object node, int type) 
	{
		switch (type) {
			case Xml.ELEMENT:
			Element elem = (Element) node;
			String namespace = elem.getNamespace();
			String prefix = null;
			String name = elem.getName();
			PrefixMap pm = elem.getPrefixMap();
			int attributeCount = elem.getAttributeCount();
			int childCount = elem.getChildCount();

			if (namespace.equals(Xml.NO_NAMESPACE)) {
				prefix = "";
			} else {
				PrefixMap map = elem.getPrefixMap();
				prefix = map.getPrefix(namespace);

				// The prefix will be the empty String if the element is in the
				// default namespace, else, it will be some prefix.
				if (prefix.equals("")) {

				// Just let the prefix be the empty String.
				} else {
					prefix = prefix + ":";
				}
			}

			stdout.print("<" + prefix + name);
			for (int i = 0; i < attributeCount; i++) {
				Attribute attrib = elem.getAttribute(i);
				stdout.print(" " + attrib.getName() + "=\'" +
					attrib.getValue() +
					"'");
			}
			stdout.print(">");

			for (int i = 0; i < childCount; i++) {
				processNode(elem.getChild(i), elem.getType(i));
			}
			stdout.print("</" + prefix + name + ">");
			break;

	         // All other node types other than Xml.ELEMENT are representedas
	         // Strings.
	
	         //case Xml.TEXT:
	            // Whitespace is counted for a Text node, so the text is
	            // bracketed to make it easier to see in the output.
	            //String text = (String) node;
	            //stdout.println("[" + text + "]");
	            //break;
			case Xml.WHITESPACE:
			stdout.print("\n");
			break;
			case Xml.TEXT:

			// Whitespace is counted for a Text node, so it will appear in the
			// output (unless there's someway to process ignorable whitespace).
			String text = (String) node;
			stdout.print(text);
			break;
			case Xml.COMMENT:
			String comment = (String) node;
			stdout.print("<!-- " + comment + " -->");
			break;
			case Xml.DOCTYPE:
			String doctype = (String) node;
			stdout.print(doctype);
			break;
			case Xml.PROCESSING_INSTRUCTION:
			String pi = (String) node;
			stdout.print("<?" + pi + "?>");
			break;
			
			default:

			// Shouldn't reach here...
			stdout.println("Unknown node type: " + type);
			break;
		}
	}

	/**
	* Command line entry point for the program.  Takes the name of the
	XML file
	* as an argument.
	* and the xpath expression
	*/
	public static void main(String[] args) 
	{
		if (args.length < 2) {
			stdout.println("Usage: java XPathDemo <xml document> <xpath expr>");
		}

		XPathDemo demo = new XPathDemo();
		demo.performDemo(args[0], args[1]);
	}
}

